@ECHO OFF

:: released 2010 March 24

:: NAME
::
:: 	run_NTBACKUP_UM.CMD - NTBACKUP supervisory script, unmanaged media mode
::
:: SYNOPSIS
::
::	run_backup_UM <media_pool> <BKS_file> <log_dir> [reporting_email]
::
:: ARGUMENTS
::
::  media_pool = RSM media pool to pull tape from
::  BKS_file = NTBACKUP selections file (.BKS)
::  log_dir = directory to put log file in
::  reporting_email = optional; email address to mail copy of log to when done
:: 
:: PREREQUISITES
::
:: SLEEP utility in PATH (Unix utility, or one of its many clones)
::
:: RSM wrappers in PATH:
::  inventory_RSM
::  view_loaded_tape
::  view_loaded_part
::  eject_RSM
::
:: If sending mail:
::  BLAT.EXE (mail tool) in PATH
::  %SMTPHOST% environment variable with name of SMTP mail relay server
::
:: NOTICES
:: 
:: Written by Ben Scott
:: Provided free-of-charge, "AS IS", WITHOUT WARRANTY.  USE AT YOUR OWN RISK.

SETLOCAL ENABLEEXTENSIONS
SET SELF=%0

:: ----------------------------------------------------------------------
:: clear internal variables

SET MYDATE=
SET MYTIME=
SET JOBID=
SET BKUP_DESC=
SET LOGFILE=
SET NTLOGFILE=
SET NTLOGDIR=
SET OLDTAPEGUID=
SET OLDTAPENAME=
SET OLDPARTNAME=
SET OLDPARTGUID=

:: ----------------------------------------------------------------------
:: command arguments

SET POOL=%1
SHIFT
SET BKS=%1
SHIFT
SET LOGDIR=%1
SHIFT
SET MAILTO=%1
SHIFT

IF NOT DEFINED POOL (
	ECHO Missing argument
	ECHO Specify RSM media pool name as first argument
	GOTO early_error
	)

IF NOT DEFINED BKS (
	ECHO Missing argument
	ECHO Specify BKS file as second argument
	GOTO early_error
	)

IF NOT EXIST "%BKS%" (
	ECHO Specified BKS file does not exist: %BKS%
	GOTO early_error
	)

IF NOT DEFINED LOGDIR (
	ECHO Missing argument
	ECHO Specify log directory as third argument
	GOTO early_error
	)

IF NOT EXIST "%LOGDIR%" (
	ECHO Specified log directory does not exist: %LOGDIR%
	GOTO early_error
	)

IF DEFINED MAILTO IF NOT DEFINED SMTPHOST (
	ECHO Email report requested but %%SMTPHOST%% not defined
	GOTO early_error
	)

:: ----------------------------------------------------------------------
:: build job ID

:: put current date and time into a variable
:: this serves as sort of a "job ID" for this run
:: format YYYY-MM-DD-HH-MM-SS

FOR /F "tokens=2-4 delims=/ " %%a IN ("%DATE%") DO SET MYDATE=%%c-%%a-%%b
FOR /F "tokens=1-3 delims=.: " %%a IN ("%TIME%") DO SET MYTIME=%%a-%%b-%%c
SET JOBID=%MYDATE%-%MYTIME%

:: ----------------------------------------------------------------------
:: derived configuration

:: FIXME we try and tell NTBACKUP to do use this, but it appears to ignore us
SET BKUP_DESC=%COMPUTERNAME%_%JOBID%

SET NTLOGDIR=%USERPROFILE%\Local Settings\Application Data\Microsoft\Windows NT\NTBackup\data\

:: this will be *our* log file, not the one NTBACKUP creates
SET LOGFILE=%LOGDIR%\%JOBID%.LOG

:: ----------------------------------------------------------------------
:: begin log file

CALL :logmsg Script started
CALL :logmsg SELF=%SELF%
CALL :logmsg CD=%CD%
CALL :logmsg COMPUTERNAME=%COMPUTERNAME%
CALL :logmsg JOBID=%JOBID%
CALL :logmsg POOL=%POOL%
CALL :logmsg BKS=%BKS%
CALL :logmsg MAILTO=%MAILTO%

:: ----------------------------------------------------------------------
:: check external variables

IF NOT DEFINED RSM_LIB_GUID (
	CALL :logmsg Set %%RSM_LIB_GUID%% to GUID of media library [tape drive]
	GOTO finish
	)

IF NOT DEFINED RSM_INV_WAIT (
	CALL :logmsg Set %%RSM_INV_WAIT%% to time [seconds] to wait for RSM to do an inventory
	GOTO finish
	)

:: ----------------------------------------------------------------------
:: make sure required tools are available

:: RSM wrapper scripts

CALL :require_program inventory_RSM.CMD
IF ERRORLEVEL 1 GOTO finish

CALL :require_program view_loaded_tape.CMD
IF ERRORLEVEL 1 GOTO finish

CALL :require_program view_loaded_part.CMD
IF ERRORLEVEL 1 GOTO finish

CALL :require_program eject_RSM.CMD
IF ERRORLEVEL 1 GOTO finish

:: sleep (pause for N seconds) (Unix utility, or a compatible clone)
CALL :require_program SLEEP.EXE
IF ERRORLEVEL 1 GOTO finish

:: if mail is requested, we need BLAT to send it
IF DEFINED MAILTO (
	CALL :require_program BLAT.EXE
	IF ERRORLEVEL 1 GOTO finish
	)

:: ----------------------------------------------------------------------
:: get tape info and log it

CALL :logmsg Telling RSM to do an inventory...
CALL inventory_RSM

CALL :logmsg Pausing %RSM_INV_WAIT% seconds...
sleep %RSM_INV_WAIT%

CALL :logmsg Checking loaded tape...

FOR /F "usebackq" %%I IN (`CALL view_loaded_tape guid`) DO SET OLDTAPEGUID=%%I
FOR /F "usebackq" %%I IN (`CALL view_loaded_tape name`) DO SET OLDTAPENAME=%%I

:: make sure we found a tape
IF NOT DEFINED OLDTAPEGUID (
	CALL :logmsg There does not appear to be a tape loaded!
	GOTO finish
	)

:: logical tape partition (side)
FOR /F "usebackq" %%I IN (`CALL view_loaded_part guid`) DO SET OLDPARTGUID=%%I
FOR /F "usebackq" %%I IN (`CALL view_loaded_part name`) DO SET OLDPARTNAME=%%I

CALL :logmsg Tape GUID: %OLDTAPEGUID%
CALL :logmsg Tape name: %OLDTAPENAME%
CALL :logmsg Side GUID: %OLDPARTGUID%
CALL :logmsg Side Name: %OLDPARTNAME%

:: ----------------------------------------------------------------------
:: run the actual backup

CALL :logmsg Starting NTBACKUP...

NTBACKUP BACKUP "@%BKS%" /UM  /P %POOL% /V:yes /R:no /HC:on /L:S /M normal /J "%BKUP_DESC%" /N "%OLDTAPENAME%" /D "%BKUP_DESC%"

CALL :logmsg NTBACKUP exited, ERRORLEVEL=%ERRORLEVEL%

:: ----------------------------------------------------------------------
:: process log file from NTBACKUP

CALL :logmsg Processing NTBACKUP log file...

FOR /F "Tokens=*" %%a IN ('DIR "%NTLOGDIR%\backup*.log" /b /o:d') DO (SET NTLOGFILE=%%a)
CALL :logmsg NTLOGFILE=%NTLOGFILE%
CALL :logmsg Appending NTBACKUP log file to our log file...
ECHO ============================================================ >> "%LOGFILE%"
TYPE "%NTLOGDIR%\%NTLOGFILE%" >> "%LOGFILE%"
ECHO ============================================================ >> "%LOGFILE%"

:: ----------------------------------------------------------------------
:: eject tape

CALL :logmsg Ejecting media...
CALL eject_RSM

:: ----------------------------------------------------------------------
:: post-run operations and cleanup

:: if things start get going but then fall apart, GOTO here rather than EOF
:finish

IF DEFINED MAILTO CALL :email_log_file

CALL :logmsg SELF=%SELF%
CALL :logmsg Script ending

:: end script
EXIT /B

:: ======================================================================
:: subroutines

:: ----------------------------------------------------------------------
:email_log_file

CALL :logmsg Emailing log file...
BLAT "%LOGFILE%" ^
	-server %SMTPHOST% ^
	-s "%COMPUTERNAME% NTBACKUP report %JOBID%" ^
	-f "%USERNAME%@%COMPUTERNAME%" ^
	-to "%MAILTO%"
:: return to CALLer
EXIT /B

:: ----------------------------------------------------------------------
:early_error
:: GOTO'ed for early errors/aborts, before logging is working
ECHO Error during initialization!  Cannot continue!
ECHO %SELF% aborting
:: exit script
EXIT /B

:: ----------------------------------------------------------------------
:logmsg
:: log a message to file and screen
SET LOGMSGTIME=[%DATE% %TIME%]
ECHO %LOGMSGTIME% %*
ECHO %LOGMSGTIME% %* >> "%LOGFILE%"
:: return to CALLer
EXIT /B

:: ----------------------------------------------------------------------
:require_program
:: makes sure a program file exists in PATH
:: returns success if true, failure if false
:: we cannot do the actual script abort here (would get interpreted as subroutine CALL return)
SET require_program_test_path=
FOR %%i IN (%1) DO SET require_program_test_path=%%~$PATH:i
IF NOT DEFINED require_program_test_path (
	CALL :logmsg Missing required tool: %1
	EXIT /B 1
	)
EXIT /B 0

:: ======================================================================
:: END OF FILE
:: ======================================================================
